"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializeExpression = void 0;
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("../utils");
const method_1 = require("./method");
/**
 * Serialize request CallExpression
 * @param node
 */
const serializeExpression = (node, checker) => {
    const targetNode = utils_1.getLeafCallExpression(node);
    const url = getUrlFromArguments(targetNode, checker);
    const customResponseInterface = getCustomResponseInterface(targetNode, checker);
    if (!customResponseInterface) {
        utils_1.logError(`url: ${url}, can not find customResponseBody`);
        return null;
    }
    const typeArgumentInterface = getTypeArgumentInterface(targetNode, checker);
    return {
        url,
        responseBody: getResponseBody(customResponseInterface, typeArgumentInterface),
    };
};
exports.serializeExpression = serializeExpression;
/**
 * Get the url value through the first parameter of request
 * @param node
 * @param checker
 */
const getUrlFromArguments = (node, checker) => {
    const urlExpression = node.arguments[0];
    if (typescript_1.default.isPropertyAccessExpression(urlExpression)) {
        // Assign value via property
        return processPropertyAccessExpression(urlExpression, checker);
    }
    else if (typescript_1.default.isStringLiteral(urlExpression)) {
        // Assign value via string
        return utils_1.getStringLiteralValue(urlExpression);
    }
    return '';
};
const processPropertyAccessExpression = (node, checker) => {
    const symbol = checker.getSymbolAtLocation(node);
    if (symbol) {
        const declaration = symbol.valueDeclaration;
        if (!typescript_1.default.isPropertyAssignment(declaration) && !typescript_1.default.isEnumMember(declaration)) {
            return '';
        }
        if (declaration.initializer && typescript_1.default.isStringLiteral(declaration.initializer)) {
            return utils_1.getStringLiteralValue(declaration.initializer);
        }
    }
    return '';
};
/**
 * Assemble the response body
 * @param responseBody
 * @param generic
 */
const getResponseBody = (responseBody, generic) => {
    transformResponseBody(responseBody, generic);
    return formatInterface(responseBody);
};
const transformResponseBody = (responseBody, generic) => {
    responseBody.properties = responseBody.properties?.map((item) => {
        if (item.kind === typescript_1.default.SyntaxKind.TypeReference &&
            responseBody.generics?.indexOf(item.value) !== -1) {
            item.value = generic;
        }
        return item;
    });
};
/**
 * Format interface to Record
 * @param entry
 */
const formatInterface = (entry) => {
    // process <MResult[]>
    if (Array.isArray(entry)) {
        return [formatInterface(entry[0])];
    }
    let formatted = {};
    entry.properties?.forEach(({ key, kind, value }) => {
        if (typescript_1.default.SyntaxKind.ArrayType === kind && typeof value === 'string') {
            formatted[key] = `${value}[]`;
        }
        else if (typescript_1.default.SyntaxKind.ArrayType === kind) {
            formatted[key] = [formatInterface(value)];
        }
        else if (typescript_1.default.SyntaxKind.TypeReference === kind && typeof value !== 'string') {
            formatted[key] = formatInterface(value);
        }
        else {
            formatted[key] = value;
        }
    });
    return formatted;
};
/**
 * Get CustomResponse corresponding to Interface
 * @param node
 */
const getCustomResponseInterface = (node, checker) => {
    // Get the symbol corresponding to the method in the request
    const symbol = checker.getSymbolAtLocation(node.expression);
    if (!symbol) {
        return null;
    }
    // Get the valueDeclaration in the symbol
    const valueDeclaration = symbol.valueDeclaration;
    if (!typescript_1.default.isMethodDeclaration(valueDeclaration)) {
        return null;
    }
    // Get the return fetch() part of the method
    const expression = method_1.getMethodEntry(valueDeclaration).expressions[0];
    if (!typescript_1.default.isCallExpression(expression)) {
        return null;
    }
    // Get the signature of the fetch method
    const signature = checker.getResolvedSignature(expression);
    // Get the fetch declaration
    const declaration = signature?.getDeclaration();
    if (declaration &&
        (typescript_1.default.isMethodDeclaration(declaration) ||
            typescript_1.default.isArrowFunction(declaration) ||
            typescript_1.default.isFunctionDeclaration(declaration))) {
        // Get R = ResponseBody<T> in the second TypeParamter in fetch
        const responseBody = declaration.typeParameters[1].default;
        if (responseBody) {
            return utils_1.processTypeReferenceNode(responseBody, checker);
        }
    }
    return null;
};
/**
 * Get the generic corresponding Interface in Request
 * @param node
 */
const getTypeArgumentInterface = (node, checker) => {
    const typeArgument = node.typeArguments[0];
    if (typescript_1.default.isTypeReferenceNode(typeArgument)) {
        return utils_1.processTypeReferenceNode(typeArgument, checker);
    }
    else if (typescript_1.default.isArrayTypeNode(typeArgument) && typescript_1.default.isTypeReferenceNode(typeArgument.elementType)) {
        // just like <MResult[]>
        return [utils_1.processTypeReferenceNode(typeArgument.elementType, checker)];
    }
    // <string> || <string[]>
    return typeArgument.getText();
};
