"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializeInterface = void 0;
const typescript_1 = __importDefault(require("typescript"));
const constants_1 = require("../constants");
const utils_1 = require("../utils");
/**
 * Get generic types in interface
 * @param paramDeclarations
 */
const getGeneric = (paramDeclarations) => paramDeclarations.map((declaration) => utils_1.getIdentifierText(declaration.name));
/**
 * Get members in interface
 * @param members
 */
const eachMembers = (members, checker) => {
    let properties = new Array();
    members.forEach((typeElement) => {
        if (typescript_1.default.isPropertySignature(typeElement)) {
            properties.push(serializeProperty(typeElement, checker));
        }
        else if (typescript_1.default.isIndexSignatureDeclaration(typeElement)) {
            properties.push(serializeIndexable(typeElement, checker));
        }
    });
    return properties;
};
/**
 * Serialize each property and its corresponding type
 * @param signature
 */
const serializeProperty = (signature, checker) => {
    const typeNode = signature.type;
    if (!typeNode) {
        return { key: '' };
    }
    return {
        key: typescript_1.default.isIdentifier(signature.name) ? utils_1.getIdentifierText(signature.name) : '',
        kind: typeNode.kind,
        value: serializePropertyValue(typeNode, checker),
    };
};
/**
 * Serialize indexable types
 * @param signature
 * @param checker
 */
const serializeIndexable = (signature, checker) => {
    const typeNode = signature.type;
    return {
        key: constants_1.INDEXABLE_TYPE,
        kind: typeNode.kind,
        value: serializePropertyValue(typeNode, checker),
    };
};
/**
 * Serialize property value
 * When the property type is other interface, call serializeInterface again
 * @param node
 * @param checker
 */
const serializePropertyValue = (node, checker) => {
    if (typescript_1.default.isTypeReferenceNode(node)) {
        return utils_1.processTypeReferenceNode(node, checker);
    }
    else if (typescript_1.default.isArrayTypeNode(node)) {
        return serializePropertyValue(node.elementType, checker);
    }
    else if (typescript_1.default.isLiteralTypeNode(node)) {
        const type = checker.getTypeAtLocation(node);
        return type.isStringLiteral() || type.isNumberLiteral() ? type.value : node.getText();
    }
    return node.getText();
};
/**
 * process interface extends case
 * @param nodeArray
 */
const serializeHeritageClause = (nodeArray, checker) => {
    const superNames = new Array();
    let properties = new Array();
    nodeArray.forEach((node) => {
        const entry = utils_1.processTypeReferenceNode(node.expression, checker);
        if (typeof entry !== 'string') {
            superNames.push(entry.name || '');
            properties = properties.concat(entry.properties || []);
        }
    });
    return {
        superNames,
        properties,
    };
};
/**
 * serialize interface
 * @param node
 */
const serializeInterface = (node, checker) => {
    const name = utils_1.getIdentifierText(node.name);
    const entry = {
        name,
        properties: eachMembers(node.members, checker),
    };
    if (node.typeParameters) {
        entry.generics = getGeneric(node.typeParameters);
    }
    // Process the extends in the interface, because there is no implements keyword in the interface, so here when there is heritageClauses,
    // Use the first one directly to be extends
    if (node.heritageClauses) {
        const { superNames, properties } = serializeHeritageClause(node.heritageClauses[0].types, checker);
        entry.extends = superNames;
        entry.properties = properties.concat(entry.properties || []);
    }
    return entry;
};
exports.serializeInterface = serializeInterface;
