"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockDataResolver = void 0;
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = require("fs");
const lodash_isequal_1 = __importDefault(require("lodash.isequal"));
const node_watch_1 = __importDefault(require("node-watch"));
const compiler_1 = require("../compiler");
const constants_1 = require("../constants");
const mock_1 = require("../generator/mock");
const utils_1 = require("../utils");
class MockDataResolver {
    constructor(opts) {
        const mockDir = utils_1.joinPath(constants_1.ROOT, opts.basePath, constants_1.MOCK_DIR);
        this.mockFilePath = utils_1.joinPath(mockDir, constants_1.MOCK_DATA_FILE);
        this.mockStructurePath = utils_1.joinPath(mockDir, constants_1.MOCK_STRUCTURE_FILE);
        this.opts = opts;
        this.originMockData = this.getDataFromMockFile();
        this.originStructure = this.getStructureFromJson();
        this.watchMockFile();
        this.watchRequestFile();
    }
    getOrGenerateData() {
        if (this.originMockData.length === 0) {
            const structure = this.getStructureFromFiles();
            const mockData = mock_1.generateMockData(structure, new Map(), []);
            if (this.opts.mockFile) {
                this.createStructureFile(structure);
                this.createMockFile(mockData);
                this.watchMockFile();
                this.watchRequestFile();
            }
            this.originMockData = mockData;
        }
        return this.originMockData;
    }
    createMockFile(mockData) {
        utils_1.logInfo('create mock json file');
        utils_1.createJsonFile(this.mockFilePath, mockData);
    }
    createStructureFile(structure) {
        utils_1.logInfo('create structure json file');
        utils_1.createJsonFile(this.mockStructurePath, structure);
    }
    getDataFromMockFile() {
        return JSON.parse(utils_1.readFile(this.mockFilePath) || '[]');
    }
    /**
     * Get structure from specified file through ast
     */
    getStructureFromFiles() {
        const files = utils_1.getFilesFromPathByRule('**/*.ts', utils_1.joinPath(constants_1.ROOT, this.opts.basePath));
        const structure = compiler_1.compileClass(files, this.opts);
        // filter if item is null
        return structure.filter((item) => !!item);
    }
    /**
     * Get the structure from structure.json
     */
    getStructureFromJson() {
        return JSON.parse(utils_1.readFile(this.mockStructurePath) || '[]');
    }
    watchMockFile() {
        if (!fs_1.existsSync(this.mockFilePath) || !this.opts.watchFile) {
            return;
        }
        node_watch_1.default(this.mockFilePath, (event) => {
            if (event === 'update') {
                utils_1.logInfo('Update the response mock data');
                const mockData = (this.originMockData = this.getDataFromMockFile());
                utils_1.eventHub.pub('UPDATE_MOCK_DATA', mockData);
            }
        });
    }
    watchRequestFile() {
        const path = utils_1.joinPath(constants_1.ROOT, this.opts.basePath);
        if (!fs_1.existsSync(path) || !this.opts.watchFile) {
            return;
        }
        node_watch_1.default(path, {
            recursive: true,
            // ignore mock data and mock structure files
            filter: (fileName) => {
                return (fileName.indexOf(constants_1.MOCK_DATA_FILE) === -1 || fileName.indexOf(constants_1.MOCK_STRUCTURE_FILE) === -1);
            },
        }, (event) => {
            if (event === 'update') {
                const structure = this.getStructureFromFiles();
                const differences = this.getStructureDifferences(this.originStructure, structure);
                if (differences.length === 0) {
                    return;
                }
                utils_1.logInfo(`Different structures are monitored: '${chalk_1.default.red(differences.join(','))}', regenerate mock file`);
                const mockData = mock_1.generateMockData(structure, mockData2Map(this.originMockData), differences);
                this.originMockData = mockData;
                this.originStructure = structure;
                if (this.opts.mockFile) {
                    this.createStructureFile(structure);
                    this.createMockFile(mockData);
                }
                else {
                    utils_1.eventHub.pub('UPDATE_MOCK_DATA', mockData);
                }
            }
        });
    }
    /**
     * Get the difference in two structures
     * @param originStructure
     * @param structure
     */
    getStructureDifferences(originStructure, structure) {
        const originMap = expressionArray2Map(originStructure);
        const currentMap = expressionArray2Map(structure);
        const differences = new Array();
        currentMap.forEach((value, key) => {
            if (originMap.has(key)) {
                if (!lodash_isequal_1.default(originMap.get(key), currentMap.get(key))) {
                    differences.push(key);
                }
            }
            else {
                // When it does not exist in the original structure, it is proved to be newly added and directly added
                differences.push(key);
            }
        });
        return differences;
    }
}
exports.MockDataResolver = MockDataResolver;
const expressionArray2Map = (expressions) => {
    let result = new Map();
    expressions.forEach((item) => {
        result.set(item.url, item.responseBody);
    });
    return result;
};
const mockData2Map = (list) => {
    const result = new Map();
    list.forEach((item) => {
        result.set(item.url, item);
    });
    return result;
};
