"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLeafCallExpression = exports.isRequestExpression = exports.processTypeReferenceNode = exports.getStringLiteralValue = exports.toString = exports.getIdentifierText = exports.getSourceFiles = void 0;
const typescript_1 = __importDefault(require("typescript"));
const interface_1 = require("../compiler/interface");
const constants_1 = require("../constants");
const getSourceFiles = (files, opts) => {
    const program = typescript_1.default.createProgram(files, opts);
    return {
        sourceFiles: program.getSourceFiles(),
        checker: program.getTypeChecker(),
    };
};
exports.getSourceFiles = getSourceFiles;
/**
 * Get the value of escapedText in Identifier
 * @param identifier
 */
const getIdentifierText = (identifier) => identifier ? identifier.escapedText.toString() : '';
exports.getIdentifierText = getIdentifierText;
/**
 * remove the `'` or `"`
 * @param value
 */
const toString = (value) => {
    return value.replace(/\"/g, '').replace(/\'/g, '');
};
exports.toString = toString;
/**
 * Get the actual value of StringLiteral
 * @param node
 */
const getStringLiteralValue = (node) => exports.toString(node.getText());
exports.getStringLiteralValue = getStringLiteralValue;
/**
 * TypeReference to interface or generic string
 * @param node
 * @param checker
 */
const processTypeReferenceNode = (node, checker) => {
    const declaration = getDeclaration(node, checker);
    if (typescript_1.default.isInterfaceDeclaration(declaration)) {
        return interface_1.serializeInterface(declaration, checker);
    }
    else if (typescript_1.default.isTypeParameterDeclaration(declaration)) {
        // process like T = any
        return exports.getIdentifierText(declaration.name);
    }
    return declaration.getText();
};
exports.processTypeReferenceNode = processTypeReferenceNode;
const getDeclaration = (node, checker) => {
    const type = checker.getTypeAtLocation(node);
    const symbol = type.symbol || type.aliasSymbol;
    const declarations = symbol?.getDeclarations();
    return declarations[0];
};
/**
 * Determine whether CallExpression is a Request by METHODS
 * @param node
 */
const isRequestExpression = (node) => {
    const targetNode = exports.getLeafCallExpression(node);
    return constants_1.METHODS.indexOf(getExpressionName(targetNode)) !== -1;
};
exports.isRequestExpression = isRequestExpression;
/**
 * Recursively find the CallExpression of the leaf node in the AST
 *
 * such as:
 * The AST structure correspondence of `Request.get().then().then()`:
 *
 * CallExpression -- Request.get().then().then()
 *  PropertyAccessExpression
 *    CallExpression -- Request.get().then()
 *      PropertyAccessExpression
 *        CallExpression -- Request.get()
 *          PropertyAccessExpression
 *          TypeReference
 *
 * @param node
 */
const getLeafCallExpression = (node) => {
    const nodeExpression = node.expression;
    if (typescript_1.default.isPropertyAccessExpression(nodeExpression) &&
        typescript_1.default.isCallExpression(nodeExpression.expression)) {
        return exports.getLeafCallExpression(nodeExpression.expression);
    }
    return node;
};
exports.getLeafCallExpression = getLeafCallExpression;
const getExpressionName = (node) => {
    const expression = node.expression;
    if (typescript_1.default.isPropertyAccessExpression(expression) && typescript_1.default.isIdentifier(expression.name)) {
        return exports.getIdentifierText(expression.name).toLocaleLowerCase();
    }
    return '';
};
